﻿# Fixme: these imports are made this way to make pyinstaller work.
#  Need to find another way for import migration in pyinstaller.
import dependency_injector.containers as containers
import dependency_injector.providers as providers

from environment.checks import PlatformEnvironment
from services.healthcheck import HealthCheck
from utils.secure import HealthAuthorization, JWTAuthorization, OTPAuthorization


class Container(containers.DeclarativeContainer):
    # Agent TODO: If possible, add logger and params.url in depends & Add in handlers
    # wiring_config = containers.WiringConfiguration(modules=["handlers.base"])
    #
    # agent_service = providers.Factory(AgentService)

    health_check_auth = providers.Singleton(HealthAuthorization)

    # STAL
    if PlatformEnvironment.is_linux():
        from constants.linux.constants import ConstStal
        from services.meta.stal import MetaService
        from services.stal import StalService

        # Config for modules
        wiring_config = containers.WiringConfiguration(
            modules=[
                "handlers.stal",
                "handlers.meta",
                "handlers.auth",
                "handlers.base",
                "core.secure",
                "utils.health_check",
            ]
        )

        # Auth
        jwt_auth = providers.Singleton(JWTAuthorization)

        # constants
        service_constants = providers.Singleton(ConstStal)

        # Services
        service = providers.Singleton(StalService, const=service_constants)
        # Services, META
        meta_service = providers.Singleton(MetaService, const=service_constants)

    # RDS
    elif PlatformEnvironment.is_windows():
        from constants.windows.constants import ConstRds
        from services.meta.rds import MetaService
        from services.rds import RdsService
        from utils.meta.rds import ApplicationRegEdit
        from utils.rds import WMIHandler

        # Config for modules
        wiring_config = containers.WiringConfiguration(
            modules=[
                "handlers.rds",
                "handlers.meta",
                "handlers.auth",
                "services.rds",
                "utils.rds",
                "core.secure",
                "utils.health_check",
            ]
        )

        # Auth
        jwt_auth = providers.Singleton(JWTAuthorization)

        # Utils, META
        # TODO: If possible, add a link to the ApplicationRegEdit.
        #  Need implementation with passing input `name` data to container
        # application_reg_edit = providers.Factory(
        #     ApplicationRegEdit,
        #     params=MetaPublishAppModel,
        #     name=""
        # )

        # constants
        service_constants = providers.Singleton(ConstRds)
        # Utils
        wmi_handler = providers.Factory(WMIHandler)
        # Services
        service = providers.Singleton(RdsService, wmi_handler=wmi_handler, const=service_constants)
        # Services, META.rds

        meta_service = providers.Singleton(MetaService, const=service_constants)

    health_check_service = providers.Singleton(HealthCheck, service)
    # Auth by OTP token
    otp_auth = providers.Singleton(OTPAuthorization)
