from dependency_injector.wiring import Provide, inject
from fastapi import Depends, Security
from fastapi.security import HTTPAuthorizationCredentials

from container import Container
from core.settings import config
from utils.secure import HealthAuthorization, JWTAuthorization


def get_token() -> str:
    """
    Func for get token_config implemented for ease of testing

    Returns:
        str: config token
    """
    return config.AGENT.HEALTH_CHECK_ACCESS_KEY


@inject
def check_auth(
    jwt_auth: JWTAuthorization = Depends(Provide[Container.jwt_auth]),
    auth: HTTPAuthorizationCredentials = Security(JWTAuthorization.security),
) -> str:
    jwt_auth.verify(auth)  # Check auth


@inject
def check_health_auth(
    health_auth: HealthAuthorization = Depends(Provide[Container.health_check_auth]),
    auth: HTTPAuthorizationCredentials = Security(HealthAuthorization.security),
) -> bool:
    """
    Token validation function

    Args:
        health_auth (HealthAuthorization): Class of validate token.
        auth (HTTPAuthorizationCredentials): Class models of token credential.

    Returns:
        bool: Answer of validate token
    """
    token_config = get_token()
    if token_config == "None" or not token_config:
        return True
    if auth is None:
        return False
    return health_auth.verify(auth=auth, token_config=token_config)  # Check auth
