import re

from fastapi_utils.api_model import APIModel
from pydantic import BaseModel, validator
from starlette.status import HTTP_401_UNAUTHORIZED, HTTP_500_INTERNAL_SERVER_ERROR, HTTP_501_NOT_IMPLEMENTED


class BaseDelegateModel(APIModel):
    url: str

    @validator("url")
    def validate_url(cls, value):
        if not re.match(r"^(?:http)s?://", value):
            raise ValueError("Invalid URL format")
        return value


class HealthCheckResponse(BaseModel):
    status: str
    version: str
    output: str
    description: str


class VersionCheckResponse(BaseModel):
    version: str


class ExceptionModel(BaseModel):
    detail: str
    type: str


class ExceptionModelHealth(BaseModel):
    description: str


# OpenAPI additional responses
HTTP_500_RESPONSE_MODEL = {
    HTTP_500_INTERNAL_SERVER_ERROR: {"model": ExceptionModel, "description": "Внутренняя ошибка сервера."}
}
HTTP_501_RESPONSE_MODEL = {
    HTTP_501_NOT_IMPLEMENTED: {
        "model": ExceptionModel,
        "description": "Метод не предназначен для вызова в этой операционной системе.",
    }
}
HTTP_401_RESPONSE_MODEL = {
    HTTP_401_UNAUTHORIZED: {"model": ExceptionModel, "description": "Учетные данные не прошли проверку."}
}
HTTP_401_RESPONSE_MODEL_HEALTH = {
    HTTP_401_UNAUTHORIZED: {
        "model": ExceptionModelHealth,
        "description": "Учетные данные не прошли проверку.",
    }
}
