import json
from typing import List, Optional, Union

from fastapi_utils.api_model import APIModel
from pydantic import BaseModel, validator


class StalUserModel(APIModel):
    login: str
    domain: Union[str, None] = ""
    password: str


class StalLogoutModel(APIModel):
    session_id: int


class StalRemoteAppModel(StalUserModel):
    program: str


class VersionResponse(BaseModel):
    version: Union[str, None]
    message: str


class StatusResponse(BaseModel):
    is_stal: Union[bool, None]


class Session(BaseModel):
    username: str
    session_id: int
    host_server: str
    state: str
    is_desktop: bool


class SessionsResponse(BaseModel):
    sessions: Union[List[Session], None]


class LoginUserResponse(BaseModel):
    auth_done: bool


class DisconnectUserResponse(BaseModel):
    message: str


class ShutdownUserResponse(BaseModel):
    disconnect_done: bool


class TestConnectResponse(BaseModel):
    is_connected: bool


class SessionRequestResponse(BaseModel):
    session_id: int


# FIXME [aap]: Объединить с SessionRequestResponse
#              после согласования ответа в session_id
class ProgramRequestResponse(BaseModel):
    session_id: Union[int, bool]


class AppsResponse(BaseModel):
    apps: Union[List[str], None]


class StalSessionModel(APIModel):
    username: str
    domain: Optional[str] = ""
    password: str
    data: str
    server_id: str

    @validator("data")
    def validate_data(cls, value):
        value = json.loads(value)
        for key in value:
            if key not in ["token", "program"]:
                raise ValueError("Incorrect data")
        return value
