﻿import configparser
import logging
import socket

from OpenSSL import crypto

fqdn = socket.getfqdn()
hostname = socket.gethostname()
IP = socket.gethostbyname(hostname)
PATH_TO_CONFIG = "C:/ProgramData/UVEON/Termidesk Session Agent/"
logger = logging.getLogger("ssl")


def generate_selfsigned_cert(
    key_file=None,
    cert_file=None,
    email_address="cert@uveon.ru",
    common_name=fqdn,
    country_name="RU",
    locality_name="Moscow",
    state_or_province_name="Moscow",
    organization_name="UVEON",
    organization_unit_name="ITG",
    serial_number=0,
    validity_end_in_seconds=10 * 365 * 24 * 60 * 6,  # год в секундах
):
    k = crypto.PKey()
    k.generate_key(crypto.TYPE_RSA, 2048)
    cert = crypto.X509()
    cert.get_subject().C = country_name
    cert.get_subject().ST = state_or_province_name
    cert.get_subject().L = locality_name
    cert.get_subject().O = organization_name  # noqa
    cert.get_subject().OU = organization_unit_name
    cert.get_subject().CN = common_name
    cert.get_subject().emailAddress = email_address
    cert.add_extensions(
        [
            crypto.X509Extension(b"subjectAltName", critical=False, value=",".join([f"DNS:{IP}"]).encode()),
            crypto.X509Extension(b"basicConstraints", critical=True, value=b"CA:false"),
        ]
    )

    cert.set_serial_number(serial_number)
    cert.gmtime_adj_notBefore(0)
    cert.gmtime_adj_notAfter(validity_end_in_seconds)
    cert.set_issuer(cert.get_subject())
    cert.set_pubkey(k)
    cert.sign(k, "sha512")

    key_data = crypto.dump_privatekey(crypto.FILETYPE_PEM, k).decode("utf-8")
    cert_data = crypto.dump_certificate(crypto.FILETYPE_PEM, cert).decode("utf-8")

    logger.info("Пара SSL сертификат — приватный ключ не были найдены.")
    with open(cert_file, "w", encoding="utf-8") as file:
        file.write(cert_data)
        logger.info(f"Сгенерирован самоподписанный сертификат: {cert_file}")
    with open(key_file, "w", encoding="utf-8") as file:
        file.write(key_data)
        logger.info(f"Сгенерирован приватный ключ: {key_file}")

    return key_data, cert_data


try:
    config = configparser.ConfigParser()
    config.read(f"{PATH_TO_CONFIG}session_agent.ini")

    if config["SSL"]["USE_HTTPS"]:
        logger.info("USE_HTTPS включен, генерирую сертификаты")
        path = f"{PATH_TO_CONFIG}certs"
        generate_selfsigned_cert(key_file=f"""{path}/key.pem""", cert_file=f"""{path}/cert.pem""")
    else:
        logger.info("USE_HTTPS выключен")
except OSError as error:
    logger.warning(str(error))
