import logging

from fastapi import status

import requests
from constants.common_const import ConstServicesBase as const

logger = logging.getLogger(__name__)


class AgentService:
    """class for interacting with the VM Agent"""

    def __init__(self, vm_delegate_url):
        self.vm_delegate_url = vm_delegate_url
        logger.info("AgentService: SSA accept delegation and management from VM Agent")

    def process(self):
        """Method for initiating settings
        from the SSA side after receiving the delegate signal from an VM Agent"""

        # Calling the target function of execute SSA settings
        self.setup_agent()

        # After completing the setup, we return control to the VM agent
        self.delegate_control()

    def setup_agent(self):
        """A method for execute settings from the SSA side
        after receiving the delegate signal from the agent"""
        pass

    def delegate_control(self) -> bool:
        """Delegate control back to agent after completion all actions in process.

        Returns:
            bool: Result of delegation.
        """
        log_postfix = f"request: {const.DELEGATE_SUCCESS_ANSWER} vm_delegate_url: {self.vm_delegate_url}"
        try:
            request = requests.post(
                self.vm_delegate_url,
                json=const.DELEGATE_SUCCESS_ANSWER,
                timeout=const.REQUEST_TIMEOUT,
                verify=False,
            )
            if request.status_code == status.HTTP_200_OK:
                logger.info(
                    f"AgentService: VM agent accepted delegation and management from SSA. {log_postfix}"
                )
                return True
            else:
                logger.error(
                    "AgentService: VM agent don't accept "
                    "delegation and management from SSA. "
                    f"Status_code: {request.status_code} "
                    f"{log_postfix}"
                )
                return False
        except requests.ConnectionError:
            logger.error(
                f"AgentService: VM agent don't accept delegation and management from SSA. {log_postfix}",
                exc_info=True,
            )
            return False
