from core.settings import config


class HealthCheck:
    def __init__(self, service) -> None:
        """
        For  now this class work only RDS and STAL, without META

        Args:
            service: head service
        """
        self.service = service
        self.__init_check_dict()

    def get_status_service(self) -> bool:
        """
        Call status method of service

        Returns:
            True or False
        """
        return self.service.status()

    def __init_check_dict(self) -> None:
        """In this list add methods with checks"""
        self.check_list = {
            "Get status service": self.get_status_service,
            "Check ssl certs": self.check_ssl_certs,
        }

    def __checking_list_for_errors(self, collection: dict) -> tuple:
        """
        Loop of check errors on dict with func

        Args:
            collection: dict with check func

        Returns:
            Bool and error optional
        """
        for key, func in collection.items():
            if not func():
                return False, key
        return True, ""

    def check_ssl_certs(self) -> bool:
        """
        Check all certs and return bool answer

        Returns:
            Result of checks
        """
        file_key = config.private_key_exists()
        file_cert = config.cert_exists()
        return all([file_key, file_cert])

    def status_service(self) -> tuple:
        """
        Run execute healthcheck

        Returns:
            Results of check in bool format
        """
        return self.__checking_list_for_errors(self.check_list)
